(****************************************************************************)
(* Level 0/1 Prover                                                         *)
(* Copyright (C) 2004 Alwen Tiu                                             *)
(*                                                                          *)
(* This program is free software; you can redistribute it and/or modify     *)
(* it under the terms of the GNU General Public License as published by     *)
(* the Free Software Foundation; either version 2 of the License, or        *)
(* (at your option) any later version.                                      *)
(*                                                                          *)
(* This program is distributed in the hope that it will be useful,          *)
(* but WITHOUT ANY WARRANTY; without even the implied warranty of           *)
(* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *)
(* GNU General Public License for more details.                             *)
(*                                                                          *)
(* You should have received a copy of the GNU General Public License        *)
(* along with this code; if not, write to the Free Software Foundation,     *)
(* Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA             *)
(****************************************************************************)

(* Lexical analyzer *)


structure Tokens = Tokens
open TextIO

type pos = int
type svalue = Tokens.svalue
type ('a,'b) token = ('a, 'b) Tokens.token
type lexresult = (svalue, pos) token

val pos = ref 0
val eof = fn () => Tokens.EOF(!pos, !pos)
val error = fn (e, l : int, _) => 
                output(stdOut, "line " ^ (Int.toString l) ^ " : "
		               ^ e ^ "\n")
		               
fun toInt s = foldl (fn (a,r) => ord(a) - ord(#"0")+10*r) 0 (explode s)

val str_ref = ref "";
		               
%%
%header (functor LamLexFun(structure Tokens : Lam_TOKENS));
%s COMMENT STRING NUM;
sstart=#;
		               
alpha = [A-Za-z];
alphanum = [A-Za-z0-9];
digit = [0-9];
ws = [\ \t];

%%

<INITIAL>\n     => (pos := (!pos) + 1; lex());
<INITIAL>"%"  => (YYBEGIN COMMENT; continue());
<INITIAL>{ws}+  => (lex());

<INITIAL>"&" => (Tokens.AND(!pos,!pos));
<INITIAL>"," => (Tokens.AND(!pos,!pos));
<INITIAL>";" => (Tokens.OR(!pos,!pos));
<INITIAL>"=>" => (Tokens.IMP(!pos, !pos));
<INITIAL>"=" => (Tokens.EQ(!pos,!pos));
<INITIAL>"print" => (Tokens.PRINT(!pos,!pos));
<INITIAL>"pi" => (Tokens.PI(!pos,!pos));
<INITIAL>"sigma" => (Tokens.SIGMA(!pos,!pos));
<INITIAL>"nabla" => (Tokens.NABLA(!pos,!pos));

<INITIAL>"parse" => (Tokens.PARSE(!pos,!pos));
<INITIAL>"unparse" => (Tokens.UNPARSE(!pos,!pos));

<INITIAL>":=" => (Tokens.DEF(!pos, !pos));
<INITIAL>"include" => (Tokens.INCLUDE(!pos,!pos));
<INITIAL>"exit" => (Tokens.EXIT(!pos, !pos));

<INITIAL>{digit}+ => (Tokens.ID(yytext, !pos, !pos));
<INITIAL>{alpha}{alphanum}* => (Tokens.ID(yytext,!pos,!pos));
<INITIAL>"(" => (Tokens.LPAREN(!pos,!pos));
<INITIAL>")" => (Tokens.RPAREN(!pos,!pos));
<INITIAL>"." => (Tokens.DOT(!pos, !pos));
<INITIAL>"\\" => (Tokens.BSLASH(!pos, !pos));
<INITIAL>\"			=> (YYBEGIN STRING;
				    str_ref := "";
				    continue ());

<STRING>[^"\\\n]+		=> (
					str_ref := (!str_ref) ^ yytext;
				    continue ());
<STRING>\\\n			=> 
					(pos := !pos + 1;
				    continue ());
				    
<STRING>\\n			=> (str_ref := (!str_ref) ^ "\n";
				    continue ());
				    
<STRING>\\t			=> (str_ref := (!str_ref) ^ "\t";
				    continue ());
<STRING>\\\\			=> (str_ref := (!str_ref) ^ "\\";
				    continue ());
<STRING>\\\"			=> (str_ref := (!str_ref) ^ "\"";
				    continue ());
<STRING>\"			=> (YYBEGIN INITIAL;
				    Tokens.STR(!str_ref, !pos, !pos));    

                       
<COMMENT>\n => (pos := (!pos) + 1; YYBEGIN INITIAL; continue());
<COMMENT>. => (continue());
